<?php namespace Myriad\Squire;

use Myriad\Squire\Contracts\Squire;
use Symfony\Component\HttpFoundation\File\File;

class SquireClient implements Squire {

  /**
   * @inheritdoc
   */
  public function __construct(SquireConfiguration $config) {
    $this->config = $config;
  }

  /**
   * @inheritdoc
   */
  public function get($key) {
    return $this->doRequest('GET', $key)->getBody()->detach();
  }

  /**
   * @inheritdoc
   */
  public function put($resource, $key) {
    if ($resource instanceof File) {
      $resource = $resource->openFile();
    }
    $this->doRequest('PUT', $key, [
      'body' => $resource
    ]);
  }

  /**
   * @inheritdoc
   */
  public function delete($key) {
    $this->doRequest('DELETE', $key);
  }

  /**
   * Creates and sends an authorized request
   *
   * @param string $method the http method
   * @param string $key the file key
   * @param array $options additional guzzle options
   *
   * @return \GuzzleHttp\Message\FutureResponse|\GuzzleHttp\Message\ResponseInterface|\GuzzleHttp\Ring\Future\FutureInterface|null
   */
  private function doRequest($method, $key, $options = []) {
    $defaultOptions = [
      'headers' => [
        'Authorization' => $this->getConfiguration()->getServerToken()
      ]
    ];
    $client = new \GuzzleHttp\Client();
    $request = $client->createRequest($method, $this->config->getServerUrl() . '/api/v1/files/'
                                               . $key, array_merge($defaultOptions, $options));

    return $client->send($request);
  }

  /**
   * @inheritdoc
   */
  public function image($key) {
    return new SquireImage($key, $this->getConfiguration());
  }

  /**
   * @inheritdoc
   */
  public function file($key) {
    return new SquireFile($key, $this->getConfiguration());
  }

  /**
   * @inheritdoc
   */
  public function getConfiguration() {
    return $this->config;
  }

}